'''
Mass Extinction in Evolutionary Algorithms
Author: Kaelan Engholdt
Version: 10/07/2020

Holds all of the adjustable parameters for the Extinction class held in 'mass_extinction.py'.
Consult Sections 8 and 9 of the README.txt for more information and details regarding the parameters.

Note: The user must add the code found under the newborn parameters to the initialization of their member class
      (See Sub-Section 8.9).

'''

# TODO user must copy this file and place it in the same directory as their evolutionary algorithm

# TODO user must properly adjust all of the parameters they desire to use

# extinction parameters (see Sub-Section 8.1)
extinction_on = None             # turns extinction on or off
ext_percent = 0.00               # percent of population to kill (completely random, anti-elitism not included)
ext_duration = 0                 # generations it will take to kill off ext_percent of members, value of '1' is instant
ext_kills_constant = None        # indicates if amount extinction kills is constant (True) or logarithmic (False)
ext_virulence = None             # indicates if extinction gets stronger (True) or weaker (False) as is progresses
ext_dynamics = None              # indicates if a few more members will be killed and then repopulated, turns on or off
ext_dyn_strength = 0.00          # indicates how strong population dynamics are, a value close to '0.50' is recommended
ext_end_extinction = None        # indicates if extinction turns off after reaching a later generation, turns on or off
ext_end_percent = 0.00           # percent of generations that will execute before extinction is turned off
ext_safeguard = 0                # minimum number of generations between possible extinction events
ext_print_on = None              # turns print statements on or off for extinction and repopulation information
ext_testing_on = None            # indicates if parameter sets in ext_types.py will be used (True) or user sets (False)
ext_uses_DEAP = None             # indicates if the Distributed Evolutionary Algorithms in Python (DEAP) library is used

# elitism parameters (see Sub-Section 8.2)
ext_use_elitism = None           # turns elitism on or off
ext_elite_percent = 0.00         # percent of population to consider elite (guaranteed to survive extinction)
ext_elite_target = None          # targets the members at the front (True) or end (False) of the sorted population list

# anti-elitism parameters (see Sub-Section 8.3)
ext_use_anti_elite = None        # turns anti-elitism on or off
ext_anti_percent = 0.00          # percent of population to consider anti-elite (will be killed if an extinction occurs)
ext_anti_target = None           # targets the members at the front (True) or end (False) of the sorted population list

# extinction types (see Sub-Section 8.4)
interval_extinction_on = None    # turns interval extinction on or off
prob_extinction_on = None        # turns probabilistic extinction on or off
fit_extinction_on = None         # turns fitness extinction on or off

# interval extinction parameters (see Sub-Section 8.5)
i_ext_interval = 0               # number of generations between extinction events (begins counting after repopulation)
i_ext_occurs = None              # extinction occurs (True) or simply has the chance to occur (False), turns on or off

# probabilistic extinction parameters (see Sub-Section 8.6)
p_ext_base_prob = 0.00           # base extinction probability (resets to base probability after extinction event)
p_ext_inc = 0.00                 # amount to increase p_ext_prob by when a generation passes, no extinction event occurs

# fitness extinction parameters (see Sub-Section 8.7)
f_ext_use_threshold = None       # indicates the type of fitness extinction is when members pass a fitness threshold
f_ext_fit_percent_high = 0.00    # percent of parent population required to pass the f_ext_threshold_high
f_ext_threshold_high = 0.00      # when the required members reaches this fitness value, extinction occurs
f_ext_use_stagnation = None      # indicates the type of fitness extinction is when members have not improved greatly
f_ext_fit_percent_low = 0.00     # percent of parent population required to pass the f_ext_threshold_low
f_ext_threshold_low = 0.00       # if the required members have not reached this fitness value, extinction occurs

# repopulation parameters (see Sub-Section 8.8)
repop_duration = 0               # generations for population to return to original size, value of '1' is instant
repop_constant = None            # indicates if gradual repopulation amount is constant (True) or logarithmic (False)
repop_growth = None              # indicates if the repopulation gets stronger (True) or weaker (False) as it progresses
repop_use_elites = None          # turns use of elite members (if ext_use_elitism is on) for repopulation on or off
repop_elite_percent = 0.00       # percent of members that will be generated by the elite members during repopulation
repop_use_survivors = None       # turns use of surviving members for repopulation on or off
repop_survive_percent = 0.00     # percent of members that will be generated by the current members of the population

# newborn parameters (see Sub-Section 8.9)
newborn_params_on = None         # turns newborn parameters on or off
newborn_expires = 0              # number of generations repopulated members are considered newborn members
newborn_protection_on = None     # turns protection (guaranteed to survive extinction) for newborn members on or off
# TODO user must add the following two lines of code to the initialization of their member class:
#     self.ext_is_newborn = False
#     self.ext_newborn_gens = 0


'''
All of the following parameters are alternate parameters that interact with the selection methods and the crossover
and mutation rates to reflect their effects in biological extinction events.
Consult Section 9 of the README.txt for more information.
'''

# alternate parameters (see Sub-Section 9.1)
alt_params_on = None             # turns alternate parameters on or off

# alternate selection parameters (see Sub-Section 9.2)
alt_select_on = None             # turns alternate selection parameters on or off
alt_tourn_size = 0               # how much to decrease the tournament size by

# alternate crossover parameters (see Sub-Section 9.3)
alt_xo_on = None                 # turns alternate crossover parameters on or off
alt_xo_prob = 0.00               # how much to increase the probability of crossover occurring by

# alternate mutation parameters (see Sub-Section 9.4)
alt_mut_on = None                # turns alternate mutation parameters on or off
alt_mut_prob = 0.00              # how much to increase the probability of mutation occurring by


'''
All of the following variables are untunable and should NOT be changed.
They are variables that are used by the Extinction class internally as global variables.
'''

# untunable extinction variables - DO NOT CHANGE
ext_original_size = 0            # tracks original population size before extinction event
ext_pop_dynamics = []            # holds all extra members to be killed/repopulated if population dynamics are enabled
ext_occurred = False             # indicates an extinction event occurred in the current generation
ext_in_progress = False          # indicates that an extinction event is in progress
ext_gens_tracker = 1             # tracks the generational progression of the extinction event
ext_generations = 0              # tracks thet number of generations since the advent of the last extinction event
ext_gens_zeroed = False          # indicates whether this evolutionary algorithm uses zero-indexed generations or not
ext_values = []                  # holds all birth/death values to be used during extinction/repopulation
ext_out_str = ""                 # records extinction information for writing to the log file

# untunable elitism variables - DO NOT CHANGE
ext_preserve = 0                 # tracks the number of members to be preserved by elitism

# untunable anti-elitism variables - DO NOT CHANGE
ext_terminate = 0                # tracks the number of members to be killed by anti-elitism

# untunable interval extinction variables - DO NOT CHANGE
i_ext_gens = 0                   # indicates how many generations it has been since the last repopulation finished
i_ext_hits = False               # indicates that the interval hits

# untunable probabilistic extinction variables - DO NOT CHANGE
p_ext_prob = p_ext_base_prob     # current probability of extinction (starts as equal to p_ext_base_prob)

# untunable repopulation variables - DO NOT CHANGE
repop_elite_pop = []             # holds all elite members to be used in repopulation if repop_use_elites is on
repop_in_progress = False        # indicates if the population is still repopulating itself
repop_gens_tracker = 1           # tracks gradual repopulation generational progress

# untunable newborn variables - DO NOT CHANGE
newborn_count = 0                # tracks the number of newborn members currently residing in the population
